"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const child_process_1 = require("child_process");
const path = require("path");
const server_1 = require("./server");
const nodePath = process.execPath;
const yarnPath = path.join(__dirname, "../node_modules/yarn/lib/cli.js");
function log(result) {
    // tslint:disable-next-line:no-console
    if (result.output)
        console.log(result.output);
    // tslint:disable-next-line:no-console
    if (result.error)
        console.error(result.error);
    server_1.Server.logError(result.error);
    return result;
}
exports.run = (args, callback) => {
    return new Promise((resolve, reject) => {
        args.unshift(yarnPath);
        const output = [];
        const errorBuffers = [];
        const childProcess = child_process_1.spawn(nodePath, args, { cwd: process.cwd() });
        const outputHandler = jsonLines((line) => {
            if (typeof line === "string") {
                // Line that could not be parsed as JSON.
                output.push(line + "\n");
                return;
            }
            switch (line.type) {
                case "progressFinish":
                case "progressStart":
                case "progressTick":
                    // TODO: Add support for reporting progress. Ignore for now.
                    break;
                case "info":
                case "step":
                case "tree":
                    // TODO: Decide if we want to be more verbose about these messages.
                    break;
                // TODO: Stop passing through success as JSON (we rely on this behavior for now):
                // case "success":
                default:
                    // TODO: Handle most messages and stop outputting unhandled ones.
                    output.push(JSON.stringify(line) + "\n");
                    break;
            }
        });
        childProcess.stdout.on("data", outputHandler);
        childProcess.stderr.on("data", (chunk) => {
            errorBuffers.push(chunk);
        });
        const result = () => {
            return { output: output.join(""), error: Buffer.concat(errorBuffers).toString() };
        };
        childProcess.on("error", () => reject(log(result())));
        childProcess.on("close", () => setTimeout(() => resolve(log(result())), 100));
    }).then(result => {
        if (callback) {
            callback();
        }
        return result;
    });
};
exports.install = args => {
    return exports.run(["install", ...args]);
};
exports.add = args => {
    return exports.run(["add", ...args]);
};
exports.remove = args => {
    return exports.run(["remove", ...args]);
};
exports.publish = args => {
    return exports.run(["publish", ...args]);
};
exports.outdated = args => {
    return exports.run(["outdated", ...args]);
};
exports.list = args => {
    return exports.run(["config", "list", ...args]);
};
exports.upgrade = args => {
    return exports.run(["upgrade", ...args]);
};
exports.check = args => {
    return exports.run(["check", ...args]);
};
/**
 * Takes Buffer chunks and parses each complete line as a JSON object and passes it on to the
 * provided handler. Lines that could not be parsed as JSON will be passed through as strings.
 */
function jsonLines(handler) {
    let buffer = "";
    return (chunk) => {
        const lines = (buffer + chunk.toString()).split("\n");
        if (lines.length === 1) {
            // No newline in chunk, so it's probably incomplete.
            buffer = lines[0];
            return;
        }
        buffer = lines.pop() || ""; // Non-empty if chunk did not end with newline.
        for (const line of lines) {
            if (!line)
                continue;
            try {
                const message = JSON.parse(line);
                handler(message);
            }
            catch (e) {
                handler(line);
            }
        }
    };
}
